###################################################################
package BU::Logger;
###################################################################
require 5.000;
require Exporter;
use Carp;
use File::Basename;
use POSIX qw(geteuid strftime);
use IO::Handle;

@ISA = qw(Exporter);
#@EXPORT = qw(messlog_list messlog_str die_log warn_log);
@EXPORT = qw(
LoggerInit
print_log printf_log print_stdout printf_stdout print_stderr printf_stderr
warn_log die_log
getProcessingLogFileName reopenProcessingLogFile closeProcessingLogFile

$SHOW_PROCESSING
);

my $syserr;
my $logfile, $processing_logfile;
my $logtag;
my $proc_logfile_name;

sub messlog_list {
	my ($storeToProcessLog) = shift;
	my ($format) = shift;
	my @list = @_;

	$format =~ s/%!/$syserr/g;
	$format =~ s/%\./\n/g;

	my $str = sprintf( "$format\n",@list);
	print $logfile strftime("%b %d %T $logtag $str", localtime) if $logfile;

	if ($storeToProcessLog and $processing_logfile) {
		print $processing_logfile $str;
	}

	return $str;
}

sub messlog_str ($ $) {
	my ($storeToProcessLog) = shift;
	my ($mess) = shift;

	$syserr=$! if $!;

	return messlog_list($storeToProcessLog, (split( "\n", $mess ), "\n"));
}

sub sys_die_log {
	my ($errmess)=@_;
	my $mess=messlog_str('', $errmess);

	die $mess;
	return 0;
};

sub sys_warn_log {
	my ($errmess)=@_;

	return if ($errmess =~ m/^Warning:\s*couldn't\s*stat\s*file\s*\/usr\/[a-zA-Z]+\/man!$/);
	
	my $mess=messlog_str('', $errmess);

	warn $mess;
	return 0;
};

sub die_log {
	my ($format) = shift;
	my @list = @_;
	my $tmp;

	for ($i=0; $i<=$#list; $i++) {
		($list[$i], $tmp) = split("\n", $list[$i]);
	}

	my $mess=messlog_list(1, $format, @list);

	die $mess;
	return 0;
};

sub warn_log {
	my ($format) = shift;
	my @list = @_;
	my $tmp;

	for ($i=0; $i<=$#list; $i++) {
		($list[$i], $tmp) = split("\n", $list[$i]);
	}

	my $mess=messlog_list(1, $format, @list);

	warn $mess;
	return 0;
};

sub LoggerInit {
	my ($user_def_proc_logfile_name, $user_def_sys_logfile_name) = @_;

	my $name=basename($0,'.pm','.pl');
	my $h=`hostname`;

	chomp $h;
	$logtag="$h $name:";

	$SIG{__DIE__} = \&sys_die_log;
	$SIG{__WARN__} = \&sys_warn_log;

#	my $f = "> /tmp/$name.$$.log";

	checkROOT();

	my $of = undef;
	if ($user_def_sys_logfile_name) {
		$of = $user_def_sys_logfile_name;
		open (LOGFILE , ">$of")  || die_log "Unable to open log file: $of. Reason $!";
	} else {
		my $dir='/var/log';
		$dir='/tmp' unless( -w $dir );
		$of = strftime("$dir/$name.%Y-%m.log", localtime);
		open (LOGFILE , ">>$of")  || die_log "Unable to open log file: $of. Reason $!";
	}
	$logfile = \*LOGFILE; # for compatibility with perl < 5.6
	autoflush LOGFILE 1;

	$of = $user_def_proc_logfile_name ? $user_def_proc_logfile_name : strftime("/tmp/$name.%Y-%m-%d.%H.log", localtime);
	$proc_logfile_name = $of;

	open (PROCESSING_LOGFILE, ">$of") || die_log "Unable to open log file: $of. Reason $!";
	$processing_logfile = \*PROCESSING_LOGFILE;
	autoflush PROCESSING_LOGFILE 1;

	print_log(strftime("START LOGGING: %a %b %d %H:%M:%S\n\n", localtime));
}

sub print_log {
	return unless ($processing_logfile);

	print ($processing_logfile @_);
}

sub printf_log {
	return unless ($processing_logfile);

	printf ($processing_logfile @_);
}

sub print_stdout {
	print @_ if $SHOW_PROCESSING;

	return unless ($processing_logfile);

	print ($processing_logfile @_);
}

sub printf_stdout {
	printf @_ if $SHOW_PROCESSING;

	return unless ($processing_logfile);

	printf ($processing_logfile @_);
}

sub print_stderr {
	print STDERR @_ if $SHOW_PROCESSING;

	return unless ($processing_logfile);

	print ($processing_logfile @_);
}

sub printf_stderr {
	printf STDERR @_ if $SHOW_PROCESSING;

	return unless ($processing_logfile);

	printf ($processing_logfile @_);
}

sub getProcessingLogFileName()
{
	return $proc_logfile_name;
}

sub closeProcessingLogFile()
{
	close $processing_logfile;
}

sub reopenProcessingLogFile()
{
	open (PROCESSING_LOGFILE, ">>$proc_logfile_name")
		|| die_log "Unable to open log file: $proc_logfile_name. Reason $!";
	$processing_logfile = \*PROCESSING_LOGFILE;
	autoflush PROCESSING_LOGFILE 1;
}

sub checkROOT()
{
	die_log "You must have a root permissions for execution of this utility.\n" unless (geteuid() == 0);
}

BEGIN {
$SHOW_PROCESSING = 1;
}

END {
close $logfile if $logfile;
close $processing_logfile if $processing_logfile;
}

1;

__END__

=head1 NAME

Mandatory section; should be a comma-separated list of programs or functions documented by this podpage, such as: 

PSA::Logger - The PSA perl module for 

=head1 SYNOPSIS

A short usage summary for programs and functions, which may someday be deemed mandatory. 

=head1 DESCRIPTION

Long drawn out discussion of the program. It's a good idea to break this up into subsections using the =head2 directives, like 

=head2 A Sample Subection

=head2 Yet Another Sample Subection

=head1 OPTIONS

Some people make this separate from the description. 

=head1 RETURN VALUE

What the program or function returns if successful. 

=head1 ERRORS

Exceptions, return codes, exit stati, and errno settings. 

=head1 EXAMPLES

 $file='/tmp/zzzzzzzzz';

 open( FF , "< $file") || warn_log "Couldn'n open '%s'. Reason: %!\n",
"$file\n";
 close FF || die_log "Why die? %s -- %s. Reason: %!. %s\n",
"'$file'\nbad\n","Stopped";

=head1 ENVIRONMENT

Envariables this program might care about. 

=head1 FILES

All files used by the program. You should probably use the F<> for these. 

=head1 SEE ALSO

Other man pages to check out, like man(1), man(7), makewhatis(8), or catman(8). 

=head1 NOTES

Miscellaneous commentary. 

=head1 CAVEATS

Things to take special care with; sometimes called WARNINGS. 

=head1 DIAGNOSTICS

All possible messages the program can print out--and what they mean. 

=cut BUGS

