###################################################################
package BU::MIME::Mime;
###################################################################

require 5.004;
require Exporter;

use MIME::Base64;

use BU::Init;
use BU::Logger;

use BU::Objects::ErrHandler;
@ISA = qw(BU::Objects::ErrHandler);


sub init($)
{
	my $self = shift @_;
	my ($params) = @_;

	$self->{headerReaded} = '';
	$self->{boundary} = '';
	$self->{mimeHeader} = {};
	$self->{partHeader} = {};
	$self->{isBoundaryRead} = '';	# if boundary is read by previous method
	$self->{endOfDump} = ''; # set to 1 if end of dump occured

	if (exists($params->{'file_name'})) {

		unless (open FD, $params->{'file_name'}) {
			$self->setError('Unable to open input file "' . $params->{'file_name'} . '"');
			return '';
		}

		binmode FD;

		$self->{fd} = \*FD;

	} elsif (exists($params->{'file_descriptor'})) {

		$self->{fd} = $params->{'file_descriptor'};
		unless ($self->{fd}) {
			$self->setError('Input file descriptor is null.');
			return '';
		}

	} else {
		$self->setError('File name and descriptor are not specified.');
		return '';
	}

	$self->readMIMEHeader();

#	$self->SUPER::init(@_);

	return $self;
}


sub DESTROY
{
	if (exists($self->{fd}) && $self->{fd}) {
		close $self->{fd};
	}
}


sub readMIMEHeader()
{
	my ($self) = @_;

	my $mimeHeader = $self->{mimeHeader};

	my $fd = $self->{fd};
	unless ($self->{fd}) {
		$self->setError('file descriptor is null.');
		return '';
	}

	my $header_lines = 22;
	my $check_points = 8;

	my $buff = <$fd>;
	chomp($buff); chop($buff) if $buff =~ m/\r$/;

	while ($buff && ($buff !~ m/^\s*$/) && $header_lines--) {
		print_stdout ("current line $buff") if ($debug > 0);
		if ($buff =~ m/Content-Type: multipart\/related; boundary=("|')(.*)("|')/) {
			$mimeHeader->{boundary} = $2;
		}

		if ($buff =~ m/^(Dump-Type|Level|Dumped-Psa-Version|Dumped-Psa-Build|Bu-Build|Creating-Timestamp|MIME-Version|Vendor)\s*:\s*(.*)\s*$/) {
			my $fieldName = $1;
			my $fieldValue = $2;
			print_stdout ("MIME Header checkpoint field: $fieldName (check points: $check_points)\n") if ($debug > 0);
			if (($fieldName ne 'Vendor') or ($fieldValue =~ m/^\s*SWsoft\s+Inc(\.|)\s*$/)) {
				$check_points--;
				$mimeHeader->{$fieldName} = $fieldValue if ($fieldValue);
			}
		}

		$buff = <$fd>;
		chomp($buff); chop($buff) if $buff =~ m/\r$/;
	}

	if ($check_points) {
		print_stderr ("PSA::Mime: check point must be 0, but it is equal $check_points\n") if ($debug > 0);
		$self->setError('Input file is not a valid MIME file: can not find MIME header.');
		return '';
	}

	return 1;
}


sub readPartHeader()
{
	my ($self) = @_;

	return undef if $self->{endOfDump};

	my $fd = $self->{fd};
	unless ($fd) {
		$self->setError('file descriptor is null.');
		return '';
	}

	if ($self->{headerReaded}) {
		$self->setError('Try to read next mime header without previous part content reading.');
		return '';
	}

	my $boundary = $self->{mimeHeader}->{boundary};
	unless ($boundary) {
		$self->setError('Boundary are not defined.');
		return '';
	}

	$self->{partHeader} = {};
	my $header = $self->{partHeader};

	unless ($self->{isBoundaryRead}) {
		$_ = <$fd>;
		chomp($_); chop($_) if m/\r$/;
		while (!m/^--$boundary(--){0,1}$/ && !$self->{isBoundaryRead} && !eof($fd)) {
			print_stdout("search HEADER:($_)\n") if ($debug > 4);
			$_ = <$fd>;
			chomp($_); chop($_) if m/\r$/;
		}
	}
	$self->{endOfDump} = (m/^--$boundary--$/) ? 1 : 0;

	return undef if (eof($fd) or m/^--$boundary--$/); 

	$self->{isBoundaryRead} = 1;

	$_ = <$fd>;
	chomp($_); chop($_) if m/\r$/;
	while ($_ && m/^([ \t]*[^: \t]+):[ \t]*([^; \t]+)([ \t]*|;.*)$/ && !eof($fd)) {
		print_stdout ("MIME HEADER: ($1) : ($2)\n") if ($debug > 4);
		$header->{$1} = $2;
		$_ = <$fd>;
		chomp($_); chop($_) if m/\r$/;
	}
	$self->{endOfDump} = (m/^--$boundary--$/) ? 1 : 0;

	return undef if (eof($fd) or m/^--$boundary--$/); 

	$self->{headerReaded} = 1;
	$self->{isBoundaryRead} = '';
	return 1;
}


sub skipPartContent()
{
	my ($self) = @_;
	$self->resetError();

	$self->{headerReaded} = '';

	return 1;
}


sub readPartContent($)
{
	my ($self, $fd_out) = @_;

	unless ($fd_out) {
		$self->setError('Output file descriptor is equal to null.');
		return '';
	}

	binmode $fd_out;

	unless ($self->{headerReaded}) {
		$self->setError('Mime header for the current part is not found.');
		return '';
	}

	my $boundary = $self->{mimeHeader}->{boundary};
	unless ($boundary) {
		$self->setError('Boundary is not defined.');
		return '';
	}

	my $fd = $self->{fd};

	if ($_ = <$fd>) {
		chomp($_); chop($_) if m/\r$/;
		my $encoding = $self->{partHeader}->{'Content-Transfer-Encoding'};
		printf_stdout ("\nencoding: (%s)\n", $encoding) if ($debug > 5);

		if ($encoding eq 'base64') {
			while ($_ && !m/^\s*$/) {
				print "data:($_)\n" if ($debug > 10);
				my $buff = decode_base64($_);
				print $fd_out $buff;
				$_ = <$fd>;
				chomp($_); chop($_) if m/\r$/;
			}
			$self->{isBoundaryRead} = '';
		} elsif ($encoding eq 'binary') {
			while ($_ && !m/^--$boundary(--){0,1}$/) {
				print $fd_out $_;
				$_ = <$fd>;
				chomp($_); chop($_) if m/\r$/;
			}
			$self->{isBoundaryRead} = (m/^--$boundary(--){0,1}$/) ? 1 : 0;
			$self->{endOfDump} = (m/^--$boundary--$/) ? 1 : 0;
		} else {
			$self->setError("Unknown Content-Transfer-Encoding: $encoding");
			return '';
		}
	}

	$self->{headerReaded} = '';
	return 1;
}


sub getPartContent()
{
	my ($self) = @_;

	unless ($self->{headerReaded}) {
		$self->setError('Mime header for the current part is not found.');
		return '';
	}

	my $boundary = $self->{mimeHeader}->{boundary};
	unless ($boundary) {
		$self->setError('Boundary is not defined.');
		return '';
	}

	my $fd = $self->{fd};
	my $out_buff = '';

	if ($_ = <$fd>) {
		chomp($_); chop($_) if m/\r$/;

		my $encoding = $self->{partHeader}->{'Content-Transfer-Encoding'};
		printf_stdout ("\nencoding: (%s)\n", $encoding) if ($debug > 0);

		if ($encoding eq 'base64') {
			chomp($_); chop($_) if m/\r$/;
			while ($_ && !m/^\s*$/) {
				print "data:($_)\n" if ($debug > 10);
				my $buff = decode_base64($_);
				$out_buff .= $buff;
				$_ = <$fd>;
				chomp($_); chop($_) if m/\r$/;
			}
			$self->{isBoundaryRead} = '';
		} elsif ($encoding eq 'binary') {
			while ($_ && !m/^--$boundary(--){0,1}$/) {
				$out_buff .= $_;
				$_ = <$fd>;
				chomp($_); chop($_) if m/\r$/;
			}
			$self->{isBoundaryRead} = (m/^--$boundary(--){0,1}$/) ? 1 : 0;
			$self->{endOfDump} = (m/^--$boundary--$/) ? 1 : 0;
		} else {
			$self->setError("Unknown Content-Transfer-Encoding: $encoding");
			return '';
		}
	}

	$self->{headerReaded} = '';
	return $out_buff;
}


sub getMimeHeaderField($)
{
	my ($self, $fieldName) = @_;

	my $mimeHeader = $self->{mimeHeader};

	return $mimeHeader->{$fieldName};
}


sub getPartHeaderField($)
{
	my ($self, $fieldName) = @_;

	my $partHeader = $self->{partHeader};

	return $partHeader->{$fieldName};
}


return 1;

