#####################################################
package BU::PSA::Restore::Domain;
#####################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::XML::CommonFunc;
use BU::PSA::Objects::Domain;
use BU::PSA::Objects::DomainLimits;
use BU::PSA::Restore::Mail;
use BU::PSA::Restore::DNSRec;
use BU::PSA::Restore::Database;
use BU::PSA::Restore::Hosting;
use BU::PSA::Restore::IPMapping;
use BU::PSA::Restore::DomainLevelUser;
use BU::PSA::Restore::WebApps;
use BU::PSA::Restore::DSMail;
use BU::PSA::Restore::DSMailLists;
use BU::PSA::Restore::CommonData qw(restoreReports restoreCustomButtons);
use BU::PSA::Restore::ActionLog;

@ISA = qw(Exporter);
@EXPORT = qw(restoreDomain getDumpLimitUsage);


sub restoreDomain($ $ $; $ $)
{
	my ($clientId, $clientLogin, $domainNode, $single_domain_mode, $single_domain_ip_id) = @_;

	$single_domain_mode = '' unless defined($single_domain_mode);

	my $domainName = $domainNode->getAttribute('name');
	my $domainDisplayName = u2a($domainNode->getAttribute('displayName'));

	if ($PRETTYPRINT) {
		print_sl("Domain $domainName");
	} else {
		$conf{'DOMAINS_PROCESSED_NUMBER'}++;
		printf_stdout("%s domain: '%s' [%s/%s]\n", $ProcessName, $domainName, $conf{'DOMAINS_PROCESSED_NUMBER'}, $conf{'DOMAINS_TOTAL_NUMBER'});
	}

	############################################
	### FETCH AND USE DOMAIN NAME AS IN DATABASE
	my $query = dbDoQuery('SELECT name FROM domains WHERE lower(name)=lower(' . dbQuote($domainName) . ')');
	if (my ($tmp) = $query->fetchrow_array()) {
		$domainName = $tmp;
		$domainNode->setAttribute('name', $domainName);
	}
	############################################

	$domain = new BU::PSA::Objects::Domain($clientId, $domainName);

	unless ($domain) {
		printf_stderr("Unable to create %s object (for domain %s).\n", 'BU::PSA::Domain', $domainName);
		return '';
	}

	if ($domain->error()) {
		warn $domain->getErrorMsg();
		return '';
	}
	$domain->setField('displayName', $domainDisplayName);

	my $domainId = '';
	my $isExist = $domain->Exists();
	my $oldDomainStatus = 0;
	if ($isExist) {
		$oldDomainStatus = $domain->getField('status');
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		if ($isExist) {
                     
                     #########################################################
                     #   For server-wide restore we shouldn't try to restore
                     #   domains which are currently dumped or restored by
                     #   clients because it can lead to a dump corruption.
                     #########################################################
                     if (not $single_domain_mode) {

                        ############################
                        #  Determine domain status
                        ############################
                        my $sq = dbDoQuery("SELECT status FROM domains WHERE name='$domainName'");
                        my ($domain_status) = $sq->fetchrow_array;

                        if ($domain_status & 4) {
                           ################################################################
                           #   Domain was turned off due to the backup/restore procedure.
                           #   We can't restore it from the server dump because it can 
                           #   lead to the data corruption.
                           ################################################################
                           markCollision($domainNode);

                           ##########################################################
                           #  Add it to the list, to later print on admin's console
                           ##########################################################
                           addNotRestoredDomain($domainNode);

						   return '';
                        }
			}

			$domainNode->setAttribute('exists', 'true');

			unless ($single_domain_mode) {
				my $query = dbDoQuery('SELECT login FROM clients c, domains d WHERE c.id=d.cl_id'
										. ' and d.name=' . dbQuote($domainName));
				my ($domainClientLogin) = $query->fetchrow_array if $query;

				unless ($query || $domainClientLogin) {
					printf_stderr("Unable to check whether a client owning the domain %s exists in the PSA system.\n",
									$domainName);
				} elsif ($clientLogin ne $domainClientLogin) {
					printf_stderr("Restoring for the %s domain is impossible: ".
						 "domain with such name already exists and owned by another client.\n",
						 $domainName);

					markCollision($domainNode);
					return '';
				}

				my @recsNodeList = $domainNode->xql('./dns/host[@name="' . a2u($domainDisplayName) . '."]');
				my $ip = '';
				my $i=0;
				for ($i=0; $i<=$#recsNodeList && !$ip; $i++) {
					my $recNode = $recsNodeList[$i];

					if (($recNode->getAttribute('type') eq 'A') && (u2a($recNode->getAttribute('name')) eq "$domainDisplayName.")) {
						if (my $ip_id = $recNode->getAttribute('ip_address_id')) {
							$ip = getIPAddressById($ip_id);
							last;
						}
					}
				}

				if ($ip) {
					print_stdout ("\ndomain based on ip => $ip\n") if ($debug > 0);

					my $query = dbDoQuery('SELECT val FROM dns_recs WHERE type="A"'
										. ' and host=' . dbQuote("$domainDisplayName."));

					my ($real_ip) = $query->fetchrow_array if $query;

					if ($real_ip ne $ip) {
						printf_stderr('The %s domain restoration is impossible: '.
								 'the domain already exists in PSA and have ip '.
								 "address which differ from stored ip address (%s)\n", $domainName, $ip);

						markCollision($domainNode);
						return '';				
					}
				}
			}

			$domainId = $domain->getDBId();

		} else {
			my $query = dbDoQuery('SELECT sd.name, d.name FROM domains d, subdomains sd WHERE d.id=sd.dom_id AND d.name!=' . dbQuote($domainName) . ' AND CONCAT(CONCAT(sd.name, "."), d.name)=' . dbQuote($domainName));

			unless ($query) {
				print_stderr("Unable to check domain and subdomains collisions for domain $domainName: sql error\n");
			}

			if (my ($tmpSubDomainName, $tmpDomainName) = $query->fetchrow_array()) {
				print_stderr("Name collision detected for domain '$domainName' with subdomain '$tmpSubDomainName' on domain '$tmpDomainName'\n");
				markCollision($domainNode);
				return '';
			}

			$conf{'DOMAINS_USAGE'}++;
		}

		# restore Databases
		foreach my $dbNode ($domainNode->xql('./data_bases')) {
			restoreDatabase($domainId, $domainName, $dbNode);
		}

		# restore PhysicalHosting
		if (my ($hostingNode) = $domainNode->xql('./user[@type="hosting"]')) {
			restorePhysicalHosting($domainId, $domainName, $domainDisplayName, undef, $hostingNode, $single_domain_mode, $single_domain_ip_id);
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if ($domainNode->getAttribute('collision')) {
			printf_stderr("%s domain restoration is skipped.\n", $domainName);
			return '';
		}
	
		# restore domain attributes
		$domain->setField('cr_date', $domainNode->getAttribute('cr_date')); 
		$domain->setField('dns_zone', $domainNode->getAttribute('dns_zone'));
		my %htypes = ('vrt_hst' => 1, 'std_fwd' => 1, 'frm_fwd' => 1);
		if (exists($htypes{$domainNode->getAttribute('htype')})) {
			$domain->setField('htype', $domainNode->getAttribute('htype'));
		}
		$domain->setField('dns_type', $domainNode->getAttribute('dns_type'));
		if ($isExist) {
			my $old_status = $domain->getField('status') + 0;
			my $new_status = $domainNode->getAttribute('status') + 0;
			### set 256=STATUS_EXPIRED and 4=STATUS_BACKUP_RESTORE, as it was in old status
			$new_status = ($old_status & (256 | 4)) | (($new_status & (256 | 4)) ^ $new_status);
			if ($single_domain_mode) {
				### set 2=STATUS_WITH_PARENT, as it was in old status
				### reason: client is not restored and him status leave untouched
				$new_status = ($old_status & 2) | (($new_status & 2) ^ $new_status);
			}
			$domain->setField('status', $new_status);
		} else {
			$domain->setField('status', $domainNode->getAttribute('status')); 
		}

		my $sizeNode = undef;
		if (($sizeNode) = $domainNode->xql('./size')) {
			$domain->setField('real_size', $sizeNode->getAttribute('real'));
		}

		if ($domain->error()) {
			warn $domain->getErrorMsg();
			return '';
		}

		$domain->setField('cert_rep_id', '0') unless ($domain->getField('cert_rep_id'));
		$domain->Store();

		if ($domain->error()) {
			printf_stderr("Unable to save domain %s: %s\n", $domainName, $domain->getErrorMsg());
			return '';
		}

		$domainId = $domain->getDBId();

		unless ($isExist) {
			### do not restore domain limits if domain already exists
			my $limit = $sizeNode->getAttribute('limit');
			if ($limit && ($limit ne 'unlimited')) {
				my $lim_id = $domain->getField('limits_id');
				unless ($lim_id) {
					dbDoQuery('INSERT INTO Limits(limit_name, value) VALUES("disk_space", ' . $limit . ')');
					$domain->setField('limits_id', dbGetInsertId());
					$domain->Store();
				} else {
					dbDoQuery('UPDATE Limits SET value=' . $limit . ' WHERE limit_name="disk_space" AND id=' . $lim_id);
				}
			}

			### restore action log
			restoreActionLog($domainNode, $domainId, 'domain');
		} else {
			$domain->logChanges($domainId, 'domain_update');
		}

		# restore disk usage counting parameters
		if (my ($duNode) = $domainNode->xql('./disk_usage')) {
			my $buff = XMLAttrs2SQL('REPLACE INTO disk_usage(dom_id, %s) VALUES('.$domainId.', %s)', $duNode->getAttributes());
			dbDoQuery($buff);
		}

		# restore stat
		if (my ($statNode) = $domainNode->xql('./stat')) {
			foreach my $trafficNode ($statNode->xql('./traffic')) {
				$size = $trafficNode->getAttribute('size');
				$date = $trafficNode->getAttribute('date');
				dbDoQuery('REPLACE INTO stat(dom_id, transfer, date) VALUES('
							. "$domainId, $size, " . dbQuote($date) . ')');
			}
		}

		#restore logrotation config
		if (my ($logRotateNode) = $domainNode->xql('./log_rotation')) {
			$logRotateNode->removeAttribute('id');
			my $logrot_id = $domain->getParam('logrotation_id');

			if ($logrot_id) {
				my $buff = XMLAttrs2SQL('REPLACE INTO log_rotation(id, %s) VALUES('.$logrot_id.', %s)',
										$logRotateNode->getAttributes(), 1);
				dbDoQuery($buff);
			} else {
				my $buff = XMLAttrs2SQL('INSERT INTO log_rotation(%s) VALUES(%s)',
										$logRotateNode->getAttributes(), 1);
				dbDoQuery($buff);
				$domain->setParam('logrotation_id', dbGetInsertId());
			}
		}

		# restore domain level user
		unless ($single_domain_mode) {
			if (my ($baseNode) = $domainNode->xql('./user[@type = "domain_level"]')) {
				restoreDomainLevelUser($domainId, $domainName, $baseNode);
			}
		}

		if ($single_domain_mode) {
			dbDoQuery('DELETE FROM dns_recs WHERE dom_id=' . dbQuote($domainId));
		}
		# restore DNS zone for this domain
		foreach my $dnsRecNode ($domainNode->xql('./dns/host')) {
			restoreDNSRec($domain->getDBId(), $dnsRecNode, $single_domain_ip_id);
		}

		foreach my $recNode ($domainNode->xql('./dns_masters/record')) {
			dbDoQuery("REPLACE INTO dns_masters(dom_id, ip_address) VALUES($domainId, ".dbQuote($recNode->getAttribute('ip')).')');
		}

		if (my ($paramsNode) = $domainNode->xql('./params')) {
			my $paramsNodesList = $paramsNode->getAttributes();

			for (my $i=0; $i<$paramsNodesList->getLength; $i++) {
				my $paramNode = $paramsNodesList->item($i);
				$domain->setParam($paramNode->getNodeName(), u2a($paramNode->getNodeValue()));
			}
		}

		### do not restore domain limits if domain already exists
		unless ($isExist) {
			#restore limits of this Domain
			if (my ($limitsNode) = $domainNode->xql('./limits')) {
				my $limitsNodesList = $limitsNode->getAttributes();

				my $lim_id = $domain->getField('limits_id');
				for (my $i=0; $i<$limitsNodesList->getLength; $i++) {
					my $limitNode = $limitsNodesList->item($i);
					my $name = $limitNode->getNodeName();
					my $value = u2a($limitNode->getNodeValue()) + 0;

					### restore mbox quota
					if ($name eq 'mbox_quota') {
						if ($value >= 0) {
							unless (util_exec('mailmng', '--set-domain-disk-quota',
												'--domain-name=' . $domainName,
												'--disk-quota=' . $value)) {
								printf_stderr("Unable to set mail box quota for domain '%s': %s\n", $domainName, util_exec_errormsg());
								next;
							}
						} else {
							unless (util_exec('mailmng', '--set-domain-disk-quota',
												 '--domain-name=' . $domainName,
												 '--disk-quota=-1')) {
								printf_stderr("Unable to remove mail box quota for domain '%s': %s\n", $domainName, util_exec_errormsg());
								next;
							}
						}
					}
					###### end of quota restoration

					if ($lim_id) {
						dbDoQuery("REPLACE INTO Limits(id, limit_name, value) VALUES($lim_id, \"$name\", \"$value\")");
					} else {
						dbDoQuery("INSERT INTO Limits(limit_name, value) VALUES(\"$name\", \"$value\")");
						$lim_id = dbGetInsertId();
						$domain->setField('limits_id', $lim_id);
					}
				}

				restoreActionLog($limitsNode, $domainId, 'domain_limits');
			}
		}

		#restore Parameters of this Domain
		if (my ($paramsNode) = $domainNode->xql('./ext_params')) {
			my $paramsNodesList = $paramsNode->getAttributes();

			my $param_id = $domain->getField('params_id');
			for (my $i=0; $i<$paramsNodesList->getLength; $i++) {
				my $paramNode = $paramsNodesList->item($i);
				my $name = $paramNode->getNodeName();
				my $value = u2a($paramNode->getNodeValue());
				if ($param_id) {
					dbDoQuery("REPLACE INTO Parameters(id, parameter, value) VALUES($param_id, \"$name\", \"$value\")");
				} else {
					dbDoQuery("INSERT INTO Parameters(parameter, value) VALUES(\"$name\", \"$value\")");
					$param_id = dbGetInsertId();
					$domain->setField('params_id', $param_id);
				}
			}
		}

		$domain->Store();

		# restore domain traffic if domains does not exist
		if (!$isExist || $single_domain_mode) {
			foreach my $node ($domainNode->xql('./traffic_statistic/*')) {
				if ($single_domain_mode) {
					my $query = dbDoQuery("SELECT * FROM DomainsTraffic WHERE dom_id=$domainId AND date=" . dbQuote($node->getAttribute('date')));
					if ($query && $query->fetchrow_array()) {
						# if such record already exists in db then do not replace it
						next;
					}
				}
				my $buff = XMLAttrs2SQL('REPLACE INTO DomainsTraffic(dom_id, %s) VALUES('.$domainId.', %s)', $node->getAttributes());
				dbDoQuery($buff);
			}
		}

		# restore Databases
		foreach my $dbNode ($domainNode->xql('./data_bases')) {
			restoreDatabase($domainId, $domainName, $dbNode);
		}

		#restore PhysicalHosting for this Domain
		if ($domain->getField('htype') eq 'vrt_hst') {
			if (my ($hostingNode) = $domainNode->xql('./user[@type="hosting"]')) {
				if (restorePhysicalHosting($domainId, $domainName, $domainDisplayName, $domain, $hostingNode, $single_domain_mode, $single_domain_ip_id)) {

					### if hosting is correctly restored and domain is default for some ip then
					### update ip preferences
					if (exists($conf{'DEFAULT_DOMAIN_IP_HASHREF'})) {
						my $tmp_hash = $conf{'DEFAULT_DOMAIN_IP_HASHREF'};
						if ($$tmp_hash{$domainNode->getAttribute('id')}) {
							dbDoQuery(sprintf('UPDATE IP_Addresses SET default_domain_id=%s WHERE id=%s',
											  $domain->getDBId(), dbQuote($$tmp_hash{$domainNode->getAttribute('id')})));
						}
					}
				} elsif (!$isExist) {
					$domain->setField('htype', 'none');
					$domain->Store();
				}
			}
		} elsif ($domain->getField('htype') =~ m/^(std|frm)_fwd$/) {

			#
			# Standard / Frame forwarding hosting restoration
			#
			my $redirect = '';
			if (my ($redirNode) = $domainNode->xql('./forwarding')) {
				$redirect = $redirNode->getAttribute('redirect');

				#
				# Need to know also ip_address_id
				#
				my $ipNode = getIPAddressNodeById($redirNode->getAttribute('ip_address_id'));
				my $realIpId = $ipNode->getAttribute('real_id');

				dbDoQuery(	"REPLACE INTO forwarding(dom_id, ip_address_id, redirect) ".
							"VALUES($domainId, $realIpId, ". dbQuote($redirect) . ')');

			} else {
				print_stderr("Unable to set forwarding: unable to define redirect address.\n");
			}

			if ($domain->getField('htype') eq 'frm_fwd') {
				unless (util_exec('vhostmng', '--install-frame-forwarding',
									'--vhost-name=' . $domainName, '--url=' . shellArgQuote(url_encode($redirect)))) {
					printf_stderr("Unable to restore frame forwarding for domain '$domainName': %s\n", util_exec_errormsg());
				} else {
					my $path = $conf{HTTPD_VHOSTS_D}."/$domainName";
					if (-d $path) {
						system("chown root:".$const{ADM_GROUP}." $path");
						system("chown root:".$const{ADM_GROUP}." $path/httpdocs") if (-d "$path/httpdocs");
						system("chown root:".$const{ADM_GROUP}." $path/logs") if (-d "$path/logs");
					}

					### if hosting is correctly restored and domain is default for some ip then
					### update ip preferences
					if (exists($conf{'DEFAULT_DOMAIN_IP_HASHREF'})) {
						my $tmp_hash = $conf{'DEFAULT_DOMAIN_IP_HASHREF'};
						if ($$tmp_hash{$domainNode->getAttribute('id')}) {
							dbDoQuery(sprintf('UPDATE IP_Addresses SET default_domain_id=%s WHERE id=%s',
											  $domain->getDBId(), dbQuote($$tmp_hash{$domainNode->getAttribute('id')})));
						}
					}
				}
			}
		} elsif ($domain->getField('htype') && ($domain->getField('htype') ne 'none')) {
			print_stderr("Error occured during hosting restoration: unknown hosting type.\n");
		}

		#restore reports for domain level user
		restoreReports($domainNode, $domainId, $const{'IS_DOMAIN_OWNER'});
	} # end of RESTORE if

	restoreCustomButtons($domainNode, $domainId, $const{'IS_DOMAIN_OWNER'}, 1);

	# restore MailNames
	foreach my $mailNameNode ($domainNode->xql('./user[@type = "mail"]')) {
		restoreMailName($domainId, $domainName, $mailNameNode);
	}

	# restore certificates repository
	if (my ($certsRepNode) = $domainNode->xql('./certificates_list')) {
		restoreCertsRepository($domain, $certsRepNode);
	}

	# restore domain service tomcat
	foreach my $node ($domainNode->xql('./domain_services/*')) {
		my $ds_type = $node->getAttribute('type');
		if ($ds_type eq 'tomcat') {
			my $www_prefix = undef;
			if ($domainNode->xql('./dns/host[@name = "' . a2u("www.$domainDisplayName.") . '" $and$ @cname = "' . a2u($domainDisplayName.'.') . '"]')) {
				$www_prefix = 1;
			}
			restoreDSTomcat($node, $domainId, $domainName, $www_prefix);
		} elsif ($ds_type eq 'mail') {
			restoreDSMail($node, $domain);
		} elsif ($ds_type eq 'maillists') {
			restoreDSMailLists($node, $domainId, $domainName);
		}
	}

	########################################################################
	#	Non-existing mail handling - options should be set in system files
	########################################################################
#	if ($flags{RESTORE_MODE} eq 'RESTORE') {
#		if (my ($paramsNode) = $domainNode->xql('./ext_params')) {
#		}
#	}

	if (($flags{RESTORE_MODE} eq 'CHECK')) {
		if ($isExist) {
			checkDomainLimits($domainNode, $domain);
		}
	} elsif (($domain->getField('status') > 0) && ($oldDomainStatus == 0)) {
		util_exec('domainmng', '--turn-off', "--domain-name='$domainName'");
	}

	return 1;
}


sub restoreCertsRepository
{
	my ($domain, $base_node) = @_;

	if ($PRETTYPRINT) {
		print_sl("Certificates repository");
	} else {
		printf_stdout "%s certificates repository.\n", $ProcessName;
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {

		#######################################################
		#	Mark domain's certificates with special attribute
		#######################################################
		foreach my $node ($base_node->xql('./*')) {
			if (my $certNode = getCertificateNodeById($node->getAttribute('id'))) {
				$certNode->setAttribute('repository', $domain->getField('name'));
				# hash certificate keys nodes
				foreach my $keyNode ($certNode->xql('./*')) {
	        		hashXMLNodeByCID($keyNode);
				}
			}
		}

	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		my $certs_rep_id = $domain->getField('cert_rep_id');

		foreach my $node ($base_node->xql('./*')) {
			my $certNode = getCertificateNodeById($node->getAttribute('id'));
			next unless $certNode;
			next unless ($certNode->getAttribute('restore') eq 'true');

			my $real_id = $certNode->getAttribute('real_id');
			next unless $real_id;

			if ($certs_rep_id) {
				dbDoQuery('REPLACE INTO Repository(rep_id, component_id) VALUES(' . "$certs_rep_id, $real_id" .')');
			} else {
				dbDoQuery('INSERT INTO Repository(component_id) VALUES(' . $real_id .')');
				$certs_rep_id = dbGetInsertId();
				$domain->setField('cert_rep_id', $certs_rep_id);
				$domain->Store();
			}
		}
	}

	return 1;
}


sub checkDomainLimits($ $)
{
	my ($domainNode, $domain) = @_;

	my $domLimits = new BU::PSA::Objects::DomainLimits($domain->getDBId(), $domain->getField('limits_id'));
	if ($domLimits->error()) {
		print_stderr($domLimits->getErrorMsg() . "\n");
		return '';
	}

	$domLimits->fetchList();
	if ($domLimits->error()) {
		print_stderr($domLimits->getErrorMsg() . "\n");
		return '';
	}

	my %dumpLimitsUsage = ( 
		'disk_space' 	=> 0,
		'max_traffic'	=> 0,
		'mbox_quota'	=> 0,
		'max_box'		=> 0,
		'max_redir'		=> 0,
		'max_mg'		=> 0,
		'max_resp'		=> 0,
		'max_wu'		=> 0,
		'max_db'		=> 0,
		'max_webapps'	=> 0,
		'max_maillists'	=> 0
	);

	foreach my $limit (keys %dumpLimitsUsage) {
		$dumpLimitsUsage{$limit} = getDumpLimitUsage($domainNode, $limit);
	}

	my $limit_exceed = '';
	foreach my $limit (keys %dumpLimitsUsage) {
		my $dumpLimitUsage = $dumpLimitsUsage{$limit};

		my $psaLimitUsage = $domLimits->getUsedResource($limit);
		if ($domLimits->error()) {
			printf_stderr("Unable to get used resource '%s' for domain '%s': %s\n",
						  $limit, $domain->getField('name'), $domLimits->getErrorMsg());
			croak "\n";
		}

		printf_stderr("checkDomainLimits: $limit(%s)(%s)(%s)\n", $domLimits->getValue($limit), $psaLimitUsage, $dumpLimitUsage) if ($debug > 3);

		if (($domLimits->getValue($limit) >= 0) && ($domLimits->getValue($limit) < $dumpLimitUsage + $psaLimitUsage)) {
			printf_stderr("Error: domain '%s' resource '%s' limit is %s, but required for restore: %s (in addition to %s already exists in Plesk)\n",
						  $domain->getField('name'),
						  $limit,
						  ($domLimits->getValue($limit) < 0) ? 'unlimited' : $domLimits->getValue($limit),
						  $dumpLimitUsage,
						  $psaLimitUsage
						  );
			$limit_exceed = 1;
		}
	}

	if ($limit_exceed) {
		printf_stderr("Domain '%s' will not be restored because some resource limits will be exceeded.\n", $domain->getField('name'));
		markCollision($domainNode);
		return '';
	}

	return 1;
}


sub getDumpLimitUsage($ $)
{
	my ($domainNode, $limit) = @_;

	my %limits_xql = (
		'disk_space' 	=> '',
		'max_traffic'	=> '',
		'mbox_quota'	=> '',
		'max_box'		=> './user[(@type = "mail") $and$ %s]/services[@postbox = "true"]',
		'max_redir'		=> './user[(@type = "mail") $and$ %s]/services[@redirect = "true"]',
		'max_mg'		=> './user[(@type = "mail") $and$ %s]/services[@mail_group = "true"]',
		'max_resp'		=> './user[(@type = "mail") $and$ %s]/autoresponder[@turn = "true"]',
		'max_wu'		=> './user[@type = "hosting"]/user[(@type = "web_users") $and$ %s]',
		'max_db'		=> './data_bases[%s]',
		'max_webapps'	=> './domain_services/domain_service[@type = "tomcat"]/web_app[%s]',
		'max_maillists'	=> './domain_services/domain_service[@type = "maillists"]/mail_list[%s]',
		'max_subdomains'=> './user[@type = "hosting"]/subdomains/subdomain[%s]'
	);

	my $xql_str = sprintf $limits_xql{$limit}, '($not$ @exists) $and$ ($not$ @collision) $and$ ($not$ @restore="false")';

	return 0 unless ($xql_str);

	my $arr = $domainNode->xql($xql_str);

#	print $#$arr , "($arr)($xql_str)\n";

	return $arr;
}


return 1;

