package BU::PSA::Restore::IPMapping;

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::PSA::DB;
use BU::PSA::CommonFunc;

@ISA = qw(Exporter);
@EXPORT = qw(
doParseIPsMapFile doIPsMapping getIPsListForMapping writeIPsListForMapping
makeIPAddressIndex
getIPAddressNodeById
getIPAddressNodeByIP
getIPAddressById
makeCertificatesIndex
getCertificateNodeById
);

local %ipAddressesIndex = ();
local %ipAddressesIndexByIP = ();
local %certificatesIndex = ();

sub makeIPAddressIndex
{
	my ($base_node) = @_;

	my $query = dbDoQuery('SELECT val FROM misc WHERE param="default_certificate_id"');
	my ($default_cert_id) = $query->fetchrow_array();

	foreach my $node ($base_node->xql('./*')) {
		$node->setAttribute('ssl_certificate_id', $default_cert_id) unless $node->getAttribute('ssl_certificate_id');
		$ipAddressesIndex{$node->getAttribute('id')} = $node;
		$ipAddressesIndexByIP{$node->getAttribute('ip_address')} = $node;
		print "(" . $node->getAttribute('id') . ") , (" . $node->getAttribute('ip_address') . ")\n" if $debug > 0;
	}
}


sub getIPAddressNodeById($)
{
	my ($ip_address_id) = @_;

	print "getIPAddressNodeById($ip_address_id)\n" if $debug > 0;
	my $node = $ipAddressesIndex{$ip_address_id};

	return undef unless ($node);
	return $node;
}


sub getIPAddressNodeByIP($)
{
	my ($ip_address) = @_;

	my $node = $ipAddressesIndexByIP{$ip_address};

	return undef unless ($node);
	return $node;
}


sub getIPAddressById($)
{
	my ($ip_address_id) = @_;
	return getIPAddressNodeById($ip_address_id)->getAttribute('ip_address');
}


sub makeCertificatesIndex
{
	my ($base_node) = @_;

	foreach my $node ($base_node->xql('./*')) {
		$certificatesIndex{$node->getAttribute('id')} = $node;
	}
}


sub getCertificateNodeById($)
{
	my ($cert_id) = @_;

	my $node = $certificatesIndex{$cert_id};

	return undef unless ($node);
	return $node;
}


### normalize ip or mask
sub normalizeIP($)
{
	my ($tmp) = @_;

	my @tmp_arr = split /\./, $tmp;
	for (my $i=0; $i<4; $i++) {
		$tmp_arr[$i] += 0;
	}

	return join('.', @tmp_arr);
}


sub doParseIPsMapFile($ $ $ $ $ $)
{
	my ($fname, $requiredIPsList, $externalIPList, $clientsIPPool, $sysIfacesList, $sysIPsList) = @_;

	my %ipMap = ();

   	### hash for checking case when two different ip mapped to one ip
    my %usedIPList = ();

	unless ($sysIPsList) {
		print_stderr("Can not parse ip map file: system ip addresses list is not defined.\n");
		die "\n";
	}

    print_stdout ("Read file with ip map and check system ip addresses:\n\n");

	# read map file
	unless (open (fd, $fname)) {
		print_stderr("Unable to open file '$fname'\n");
		die "\n";
	}

	my $currentIface = '';
	my $line = 0;
	my $FullHostNameSet = '';

	my $_error = undef; ### not fatal but critical errors flag

	while (<fd>) {
		$line++;

		s/#.*$//g;
		next if m/^\s*$/;

		if (m/^\s*([a-zA-Z0-9_]+)\s*:\s*$/) {
			$currentIface = $1;
			printf_stdout("Set current interface to: %s\n", $currentIface) if $debug > 0;
			next;
		}

		if (m/^\s*(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s*->\s*(([a-zA-Z0-9_]+)\s*:){0,1}\s*(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})(\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})){0,1}\s*$/) {
			my $src_ip = normalizeIP($1);
			my $ip_iface = $currentIface;
			my $ip = normalizeIP($2);
			my $ip_mask = '';

			if ($4) {
				$ip_iface = $3 if $3;
				$ip = normalizeIP($4);
				$ip_mask = normalizeIP($6) if $6;
			}

			print_stdout("mapfile line $line: ($src_ip) -> ($ip_iface) : ($ip) ($ip_mask)\n") if ($debug > 0);

			my $error = '';

			if ($ip_iface && !exists($$sysIfacesList{$ip_iface})) {
				printf_stderr("Error in map file line $line: invalid interface '%s' in line %s:%s\n", $ip_iface, $line, $_);
				$error = 1;
			}

			unless (isValidIP($src_ip)) {
				printf_stderr("Error in map file line $line: invalid ip address '%s' in line %s:%s\n", $src_ip, $line, $_);
				$error = 1;
			}

			unless (isValidIP($ip)) {
				printf_stderr("Error in map file line $line: invalid ip address '%s' in line %s:%s\n", $ip, $line, $_);
				$error = 1;
			}

			if ($ip_mask && !isValidIP($ip_mask)) {
				printf_stderr("Error in map file: invalid ip mask '%s' in line %s:%s\n", $ip_mask, $line, $_);
				$error = 1;
			}

			croak "\n" if $error;

			### ignore not used ip addresses
			if (!exists($$requiredIPsList{$src_ip}) && !exists($$externalIPList{$src_ip}) && !exists($$clientsIPPool{$src_ip})) {
				printf_stderr("Line %s in map file is ignored because source IP address '%s' is not used for restoring.\n",
							  $line, $src_ip);
				next;
			}

            my $isExternal = exists($$externalIPList{$src_ip});

			if ($ipMap{$src_ip}) {
				if ($isExternal) {
					print_stderr("WARNING: the same external ip address '$src_ip' was defined twice.\n");
				} else {
					print_stderr("Error encountered in map file line $line: the same ip address '$src_ip' was defined twice.\n");
#					croak "\n";
					$_error = 1;
					next;
				}
			}
            
			if ($isExternal) {
				$ipMap{$src_ip} = $ip;
                $usedIPList{$ip} = $src_ip;
				next; # GOTO NEXT ITERATION
            }

            ############################################################
            ### processing domain ip and ip's from clients repositories:

            my $src_ip_node = getIPAddressNodeByIP($src_ip);
			unless ($src_ip_node) {
				print_stderr("Error encountered in map file line $line: source ip $src_ip is not found in dump.\n");
#				croak "\n";
				$_error = 1;
				next;
			}

			### if target ip address already is used for some source ip then ERROR
			### except case when previous src ip and current src ip have 'shared' type
			my $prev_src_ip_node = getIPAddressNodeByIP($usedIPList{$ip});
			if (exists($usedIPList{$ip})) {
				if ($src_ip_node->getAttribute('type') ne 'shared') {
					print_stderr("Error: unable to map source ip address '$src_ip' to target ip address '$ip',"
									." because source ip address has exclusive type and shoud be mapped"
									." to free exclusive ip address,"
									." but target ip address is already used for '$usedIPList{$ip}'\n");
#					croak "\n";
					$_error = 1;
					next;
				} elsif ($prev_src_ip_node->getAttribute('type') ne 'shared') {
					print_stderr("Error: unable to map source ip address '$src_ip' to target ip address '$ip',"
								  ." because it is already used for '$usedIPList{$ip}' ip address with exclusive type.\n");
#					croak "\n";
					$_error = 1;
					next;
				}
			}

			### define owner for $src_ip
			my $src_ip_client = '';
			if ($src_ip_node->getAttribute('type') eq 'exclusive') {
				if (exists($$requiredIPsList{$src_ip})) {
					my $tmp_arr = $$requiredIPsList{$src_ip};
					($domains_for_ip, $src_ip_client) = @$tmp_arr;
				} elsif (exists($$clientsIPPool{$src_ip})) {
					my $tmp_arr = $$clientsIPPool{$src_ip};
					($src_ip_client) = $$clientsIPPool{$src_ip};
				}
			}

            if (exists($$sysIPsList{$ip})) {
                my $tmp_arr = $$sysIPsList{$ip};
                my ($target_ip_iface, $target_ip_mask, $target_ip_type, $target_ip_client) = @$tmp_arr;

				### if interface and mask is not specified then use target ip params:
				$ip_iface = $target_ip_iface unless $ip_iface;
				$ip_mask = $target_ip_mask unless $ip_mask;

				### compare params for target ip with system params 
				if ($ip_mask && ($ip_mask ne $target_ip_mask) || $ip_iface && ($ip_iface ne $target_ip_iface)) {
					printf_stderr("Error encountered in the map file: ip address '%s' is specified with".
                                  " interface '%s' and mask '%s', but".
                                  " it exists in the system with interface '%s' and mask '%s'\n",
                                  $ip, $ip_iface, $ip_mask, $target_ip_iface, $target_ip_mask);
#                    croak "\n";
					$_error = 1;
					next;
     			}

				printf_stdout("Target ip '%s' is found in the system on interface '%s' with mask: '%s'\n",
								$ip, $target_ip_iface, $target_ip_mask);

				### check type for $src_ip and for $target_ip
				### if target ip type is not defined then this ip address is exists in system but not exists in psa
                if ($target_ip_type) {
					if ($target_ip_type ne $src_ip_node->getAttribute('type')) {
						printf_stderr("Error encountered in the map file: source ip address '%s' with type '%s'" .
									  " can't be mapped to ip address '%s' with type '%s'\n",
									  $src_ip, $src_ip_node->getAttribute('type'), $ip, $target_ip_type);
#						croak "\n";
						$_error = 1;
						next;
					}
                } else {
					my @tmp_arr = ($target_ip_iface, $target_ip_mask, $src_ip_node->getAttribute('type'), $src_ip_client);
					$$sysIPsList{$ip} = \@tmp_arr;
				}

				### check for exclusive address owner of $src_ip and owner of $target_ip must be the same
				if (($src_ip_node->getAttribute('type') eq 'exclusive') && $target_ip_client && ($src_ip_client ne $target_ip_client)) {
					printf_stderr("Error encountered in the map file: exclusive source ip address '%s' with owner '%s'" .
								" can't be mapped to exclusive ip address '%s' with owner '%s'\n",
                                $src_ip, $src_ip_client, $ip, $target_ip_client);
#                    croak "\n";
					$_error = 1;
					next;
				}
			} else {
                print_stderr("Warning: target ip address '$ip' does not exist in the system.\n");

				my $error = '';
				unless ($ip_iface) {
					print_stderr("Error encountered in ip map file: undefined network interface for ip '$ip'\n");
					$error = 1;
				}
				unless ($ip_mask) {
					print_stderr("Error encountered in ip map file: undefined network mask for ip '$ip'\n");
					$error = 1;
				}
                croak "\n" if $error;

				unless ($flags{testMode}) {
					if (util_exec('ifmng', '-n', $ip_iface, $ip, $ip_mask)) {
						printf_stderr("ip address was added: %s: %s %s\n", $ip_iface, $ip, $ip_mask);
					} else {
						printf_stderr("Error: unable to add ip address $ip_iface:$ip $ip_mask in the system: %s\n",
										util_exec_errormsg());
						croak "\n";
					}
				}
				my @tmp_arr = ($ip_iface, $ip_mask, $src_ip_node->getAttribute('type'), $src_ip_client);
				$$sysIPsList{$ip} = \@tmp_arr;
			}

			$ipMap{$src_ip} = $ip;
			$usedIPList{$ip} = $src_ip;

			next;
		}


		my $reg_expr_dom = '([\w\d]([\w\d-]*[\w\d])*){1,63}(\.([\w\d]([\w\d-]*[\w\d])*){1,63})+';
		if (m/^\s*($reg_expr_dom)\s*->\s*($reg_expr_dom)\s*(|#.*)$/) {
			my $srcFHN = $1;
			my $targetFHN = $7;

			if ($FullHostNameSet) {
				print_stderr("Error in map file: FullHostName is defined twice.\n");
#				croak "\n";
				$_error = 1;
				next;
			}

			if ($flags{psaIsConfigured}) {
				$FullHostNameSet = 1;
				next;
			} else {
				$conf{'FULL_HOST_NAME'} = $targetFHN;
				print_stdout ("Change FullHostName: $srcFHN -> $targetFHN\n");
				if (my ($mailConfNode) = $psaNode->xql('./server/MailControl/conf-file[@file = "me"]')) {
					$mailConfNode->getFirstChild->setNodeValue($targetFHN);
				}
			}

			$FullHostNameSet = 1;
			next;
		}

		print_stderr("Syntax error in ip map file in line $line: $_\n");
		die "\n";
	}

	close fd;

	if ($_error) {
		print_stderr("Errors in ip mapping procedure.\n");
		croak "\n";
	}

	if ($FullHostNameSet) {
		printf_stderr("FullHostName: %s\n", $conf{'FULL_HOST_NAME'});
	}

	return \%ipMap;
}


sub doIPsMapping($ $ $ $ $ $)
{
	my ($psaNode, $requiredIPsList, $externalIPList, $ipMap, $sysIPsList, $clientsIPPool) = @_;


	# ip mapping process
	# print_stdout ("\nDo mapping process:\n\n");

	my %tmp = ();

	# map all specified in map file ip addresses and all required
	foreach my $ipNode ($psaNode->xql('./server/ip_addresses_list/*')) {
		my $ip = $ipNode->getAttribute('ip_address');

		unless ($ip =~ m/^\s*(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s*$/) {
			next;
		}
		$ip = $1;

		unless (exists($$ipMap{$ip})) {
			unless (exists($requiredIPsList->{$ip})) {
				### disable ip address restoring
				$ipNode->setAttribute('restore', 'false');
				if (my $certNode = getCertificateNodeById($ipNode->getAttribute('ssl_certificate_id'))) {
					my $used = $certNode->getAttribute('used_for_ip');
					### decrement ip usage for this certificate
					$used--;
					$certNode->setAttribute('used_for_ip', $used);
				}
				next;
			}

			### ignore not used ip addresses
			if (!exists($$requiredIPsList{$ip}) && !exists($$externalIPList{$ip}) && !exists($$clientsIPPool{$sip})) {
				next;
			}

			printf_stderr("Unable to map ip address %s: target ip is missing in map.\n", $ip);
			croak "\n";
		}

		my $target_ip = $$ipMap{$ip};
		my $tmp_arr = $$sysIPsList{$target_ip};
		my ($iface, $mask, $type, $client) = @$tmp_arr;
		### map ip address, mask and iface
		$ipNode->setAttribute('ip_address', $target_ip);
		$ipNode->setAttribute('iface', $iface);
		$ipNode->setAttribute('mask', $mask);

		### set "mapped" attribute for all ip's which was mapped
		### it is nessesary detecting ip addresses which must be restored for client ip pool 
		###### REASON:
		###### if ip address was mapped then ip pool collision was checked
		###### if ip address was not mapped then one don't have to restore in any case
		$ipNode->setAttribute('mapped', 'true');

		printf_stdout("%s -> %s: %s %s   ### %s\n", $ip, $iface, $target_ip, $mask, $type);
		$tmp{$ip} = 1;
	}

	#####################################################################################
	### map all extetnal ip addresses
	foreach my $ip (keys %$externalIPList) {
		my $target_ip = $$ipMap{$ip};
		next unless $target_ip;
		printf_stdout("%s -> %s   ### external ip address\n", $ip, $target_ip);
		foreach my $domainNode ($psaNode->xql('./user[@type="client"]/sites/domain')) {
			foreach my $node ($domainNode->xql('./dns/host[@ip="' . $ip . '"]')) {
					$node->setAttribute('ip', $target_ip);
			}
			foreach my $node ($domainNode->xql('./dns_masters/record[@ip="' . $ip . '"]')) {
				$node->setAttribute('ip', $target_ip);
			}
		}
	}

	return 1;
}


sub getIPsListForMapping()
{
	my %ipList = ();
	my %externalIPList = ();
	my %clientsIPPool = ();

    my $ext_condition = ($conf{'RESTORING_BY_CLIENTS_LIST'} || $conf{'RESTORING_BY_DOMAINS_LIST'}) ? ' $and$ @restore_by_list="true"' : '';

	### search all clients
	foreach my $clientNode ($psaNode->xql('./user[(@type="client") ' . $ext_condition . ']')) {
		next unless $clientNode;
		next unless (my ($loginNode) = $clientNode->xql('./login'));

		my $login = $loginNode->getAttribute('name');

        my $ext_condition = $conf{'RESTORING_BY_DOMAINS_LIST'} ? '[@restore_by_list="true"]' : '';
        ### search all domains for current client
		foreach my $domainNode ($clientNode->xql('./sites/domain' . $ext_condition)) {
			next unless $domainNode;
			my $domainName = $domainNode->getAttribute('name');
			my $base64DomainDisplayName = $domainNode->getAttribute('displayName');


			######################################################################################
			### search domain ip address:
			my $ip = undef;
			if (my ($node) = $domainNode->xql('./user[@type = "hosting"]/ip_address')) {
			    ### if hosting is exist then fetch hosting ip address
			    $ip = getIPAddressById($node->getAttribute('id'));
			} elsif (my ($node) = $domainNode->xql('./dns/host[(@type = "A") $and$ (@name = "' . $base64DomainDisplayName . '.")]')) {
			    ### if hosting is not exist then find dns record with type A and accordingly domain name
				if (my $ip_address_id = $node->getAttribute('ip_address_id')) {
					$ip = getIPAddressById($ip_address_id);
				}
			} elsif (my ($node) = $domainNode->xql('./dns/host[(@type = "A") $and$ (@name = "' . $domainName . '.")]')) {
			    ### if hosting is not exist then find dns record with type A and accordingly domain name
				if (my $ip_address_id = $node->getAttribute('ip_address_id')) {
					$ip = getIPAddressById($ip_address_id);
				}
			}

    		my $domains_list = '';
    		if ($ip) {
			    if (exists($ipList{$ip})) {
				my $tmp_arr = $ipList{$ip};
				($domains_list, $tmp) = @$tmp_arr;
				$domains_list .= ($domains_list ? ', ' : '') . $domainName;
			    } else {
				$domains_list = $domainName;
			    }
			    my @tmp_arr = ($domains_list, $login);
			    $ipList{$ip} = \@tmp_arr;

			    ### if such ip address is included in clients pool list for some clients
			    ### then delete it from there because it is required for domain
			    if (exists($clientsIPPool{$ip})) {
			       delete($clientsIPPool{$ip});
			    }
			}


			#####################################################################################
			### search all extetnal ip addresses
			### (ip addresses which is used but not exists in ip_addresses_list in server section)
			foreach my $node ($domainNode->xql('./dns/host[@ip!=""]')) {
			    my $ip = $node->getAttribute('ip');
			    unless ($psaNode->xql('./server/system_ip_list/address[@ip = "'.$ip.'"]')) {
                    $externalIPList{$ip} = exists($externalIPList{$ip}) ? ($externalIPList{$ip} . ', ' . $domainName) : $domainName;
			    }
			}
			foreach my $node ($domainNode->xql('./dns_masters/record')) {
			    my $ip = $node->getAttribute('ip');
			    unless ($psaNode->xql('./server/system_ip_list/address[@ip = "'.$ip.'"]')) {
                    $externalIPList{$ip} = exists($externalIPList{$ip}) ? ($externalIPList{$ip} . ', ' . $domainName) : $domainName;
			    }
			}
		}

		### search ip addresses in client ip pool
		foreach my $node ($clientNode->xql('./ip_pool/ip_address')) {
		    if (my $ipNode = getIPAddressNodeById($node->getAttribute('id'))) {
                my $ip = $ipNode->getAttribute('ip_address');

                ### check including for this ip address in $ipList
                ### if it is already included then do not add it into $clientsIPPool
                unless (exists($ipList{$ip})) {
                    $clientsIPPool{$ip} = exists($clientsIPPool{$ip}) ? ($clientsIPPool{$ip} . ', ' . $login) : $login;
                }
		    }
		}
	}

	### add client templates shared ip mapping if it is full server restoration
	if (!$conf{'RESTORING_BY_CLIENTS_LIST'} && !$conf{'RESTORING_BY_DOMAINS_LIST'}) {
		### search ip addresses in clients template ip pool
		foreach my $tpl_node ($psaNode->xql('./server/client_templates/client')) {
			my $tpl_name = $tpl_node->getAttribute('name'). '(tmpl)';

			foreach my $node ($tpl_node->xql('./ip_pool/*')) {
				if (my $ipNode = getIPAddressNodeById($node->getAttribute('id'))) {
					my $ip = $ipNode->getAttribute('ip_address');
					### check including for this ip address in $ipList
					### if it is already included then do not add it into $clientsIPPool
					unless (exists($ipList{$ip})) {
						$clientsIPPool{$ip} = exists($clientsIPPool{$ip}) ? ($clientsIPPool{$ip} . ', ' . $tpl_name) : $tpl_name;
					}
				}
			}
		}
	}

	my @arr = (\%ipList, \%externalIPList, \%clientsIPPool);

	return @arr;
}


sub writeIPsListForMapping($ $ $ $ $)
{
	my ($ipList, $externalIPList, $sysIPsList, $clientsIPPool, $ip_map_file) = @_;

	unless (open (IP_MAP_FD, ">$ip_map_file")) {
		print_stderr("Unable to open file '$ip_map_file': $!\n");
		die "\n";
	}

	print IP_MAP_FD "# ATTENTION: all not existing TARGET ip addresses will be added into your system.\n";
	print IP_MAP_FD "# Write target ip addresses for mapping.\n";
	print IP_MAP_FD "# for example:\n#\n";
	print IP_MAP_FD "# fxp0:\n";
	print IP_MAP_FD "# 192.168.1.1 -> 195.173.23.200\n";
	print IP_MAP_FD "# eth0:\n";
	print IP_MAP_FD "# 192.168.1.2 -> 193.233.1.130\n";
	print IP_MAP_FD "# 192.168.1.3 -> 193.233.1.131\n";
	print IP_MAP_FD "#\n# or simply:\n";
	print IP_MAP_FD "# 192.168.1.1 -> fxp0:195.173.23.200\n";
	print IP_MAP_FD "# 192.168.1.2 -> eth0:193.233.1.130 255.255.255.0\n";
	print IP_MAP_FD "# 192.168.1.3 -> eth0:193.233.1.131 255.255.255.0\n";
	print IP_MAP_FD "#\n".
					"# 1) if ip address exists in the system then it will be ".
					"skipped on the old interface.\n";
	print IP_MAP_FD "# 2) if netmask was not specified then used 255.255.255.0\n";
	print IP_MAP_FD "\n\n";

	unless ($flags{psaIsConfigured}) {
		print IP_MAP_FD "# the FullHostName:\n";
		print IP_MAP_FD "# " . $conf{'FULL_HOST_NAME_IN_DUMP'} . " -> # if this option is not be specified then PSA will be configured with this hostname.\n";
	}

	print IP_MAP_FD "\n\n\n#########################################################################\n";
	print IP_MAP_FD "# The following ip addresses are used for domains:\n";
	print IP_MAP_FD "#########################################################################\n";

	while (my ($src_ip, $tmp_arr) = each %$ipList) {
#    	next unless $src_ip;
    	my ($domains_for_ip, $src_ip_client) = @$tmp_arr;
    	my $src_ip_type = undef;

		if (my $src_ip_node = getIPAddressNodeByIP($src_ip)) {
		    $src_ip_type = $src_ip_node->getAttribute('type');
		} else {
		    printf_stderr("Unable to get type for ip address '%s'\n", $src_ip);
		    die "\n";
		}

		print IP_MAP_FD "\n";
		print IP_MAP_FD "# used for domains: $domains_for_ip\n" if $domains_for_ip;

		if (exists($$sysIPsList{$src_ip})) {
			my $tmp_arr = $$sysIPsList{$src_ip};
			my ($iface, $mask, $target_ip_type, $target_ip_client) = @$tmp_arr;

			print_stderr("write MAP file ($src_ip, $src_ip_type)=>($src_ip, $target_ip_type); client:($src_ip_client)=>client:($target_ip_client)\n") if $debug > 3;

			### if source ip address type eq target ip address type
			### and (source ip type is shared or owner of source ip and owner of target ip are the same client)
			### then map ip source address to target
			### else target ip address is empty
			### if $target_ip_client is not set then target ip address is used nobody
			### if target ip address type is not set then target ip address is exist in system but is not exist in PSA
			if (!$target_ip_type ||
				($src_ip_type eq $target_ip_type) &&
				(($src_ip_type eq 'shared') || !$target_ip_client || ($src_ip_client eq $target_ip_client)))
			{
			    print IP_MAP_FD "$src_ip -> $iface: $src_ip $mask    ### $src_ip_type\n";
			} else {
			    print IP_MAP_FD "$src_ip ->   # 255.255.255.0 #<-uncomment this netmask to use it as a template  ### $src_ip_type\n";
			}
		} else {
			print IP_MAP_FD "$src_ip ->   # 255.255.255.0 #<-uncomment this netmask to use it as a template  ### $src_ip_type\n";
		}
	}


	print IP_MAP_FD "\n\n\n#########################################################################\n";
	print IP_MAP_FD "# The following ip addresses are not required:\n";
	print IP_MAP_FD "#########################################################################\n";

	# print clients ip addresses
	while (my ($src_ip, $clients_list) = each %$clientsIPPool) {
	    my $src_ip_type = undef;
	    if (my $src_ip_node = getIPAddressNodeByIP($src_ip)) {
            $src_ip_type = $src_ip_node->getAttribute('type');
	    } else {
            printf_stderr("Unable to get type for ip address '%s'\n", $src_ip);
            die "\n";
	    }

	    print IP_MAP_FD "\n";
	    print IP_MAP_FD "# included in ip pool for the clients and client templates: $clients_list\n" if $clients_list;

	    if (exists($$sysIPsList{$src_ip})) {
		    my $tmp_arr = $$sysIPsList{$src_ip};
		    my ($iface, $mask, $target_ip_type, $target_ip_client) = @$tmp_arr;

		    ### if source ip address type eq target ip address type
		    ### and (source ip type is shared or owner of source ip and owner of target ip are the same client)
		    ### then map ip source address to target
		    ### else target ip address is empty
		    if (($src_ip_type eq $target_ip_type) && (($src_ip_type eq 'shared') || !$target_ip_client || ($clients_list eq $target_ip_client))) {
				print IP_MAP_FD "$src_ip -> $iface: $src_ip $mask   ### $src_ip_type\n";
		    } else {
				print IP_MAP_FD "$src_ip ->    # 255.255.255.0 #<-uncomment this netmask to use it as a template  ### $src_ip_type\n";
		    }
	    } else {
		    print IP_MAP_FD "$src_ip ->    # 255.255.255.0 #<-uncomment this netmask to use it as a template  ### $src_ip_type\n";
	    }
	}


	my $tmp = 1;
	while (my ($ip, $domains_list) = each(%$externalIPList)) {
		if ($tmp) {
			print IP_MAP_FD "\n\n\n#########################################################################\n";
			print IP_MAP_FD "# The following ip addresses are external and located on other hosts:\n";
			print IP_MAP_FD "#########################################################################\n";
			$tmp = undef;
		}

		print IP_MAP_FD "# used for domains: $domains_list\n";
		print IP_MAP_FD "# $ip ->\n";
	}

	unless (close IP_MAP_FD) {
		print_stderr("Error on writing file '$ip_map_file': $!\n");
		die "\n";
	}

	return 1;
}


return 1;

