#####################################################
package BU::PSA::Restore::SiteApp;
#####################################################

require 5.004;
require Exporter;
use Carp;

use BU::PSA::Const;
use BU::PSA::Logger;
use BU::FileFunc;
use BU::PSA::CommonFunc;
use BU::PSA::DB;
use BU::XML::CommonFunc;


@ISA = qw(Exporter);
@EXPORT = qw(restoreSiteApp);

sub restoreSiteApp
{
	my ($baseNode, $domainNode, $domainId, $domainName, $subDomainName, $subDomainId) = @_;

	my $dumped_install_prefix = u2a($baseNode->getAttribute('install_prefix'));
	my $install_dir =$conf{HTTPD_VHOSTS_D} . "/$domainName/"
					. ($subDomainName ? "subdomains/$subDomainName/" : '')
					. $baseNode->getAttribute('htdocs_directory') . '/' . $dumped_install_prefix;

	# define parent node
	my $parentNode = $baseNode->getParentNode()->getParentNode();
	if ($subDomainName) {
		if ($parentNode->getNodeName() ne 'subdomain') {
			print_stderr("FATAL error: invalid parent (subdomain) node defining for site application\n");
			die "\n";
		}
	} else {
		if (($parentNode->getNodeName() ne 'user') || ($parentNode->getAttribute('type') ne 'hosting')) {
			print_stderr("FATAL error: invalid parent(hosting) node defining for site application\n");
			die "\n";
		}
	}

	# check accordable SiteAppPackage installation
	my $app_pkg_id = $baseNode->getAttribute('app_id');
	my ($pkgNode) = $psaNode->xql('./server/site_app_packages/*[@id="'.$app_pkg_id.'"]');
	unless ($pkgNode) {
		print_stderr("Site application (with id='$app_pkg_id' in dump) will not be restored: unable to check package installation for site application.\n");
		&setRestoreFlags($baseNode, $parentNode, $domainNode, {'restore' => 'false'});
		return;
	}
	my $appName = $pkgNode->getAttribute('name');

	if ($subDomainName) {
		printf_stdout "%s site application '%s' for subdomain '%s' on domain '%s'\n", $ProcessName, $appName, $subDomainName, $domainName;
	} else {
		printf_stdout "%s site application '%s' for domain '%s'\n", $ProcessName, $appName, $domainName;
	}

	if ($flags{RESTORE_MODE} eq 'CHECK') {
		if ($pkgNode->getAttribute('collision') eq 'true') {
			printf_stderr("Unable to restore site application '$appName' for %s domain '$domainName' because accordable package conflicts with some one already installed.\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
			&setRestoreFlags($baseNode, $parentNode, $domainNode, {'collision' => 'true'});
			return undef;
		} elsif (!$pkgNode->getAttribute('real_id') && ($pkgNode->getAttribute('restore') eq 'false')) {
			printf_stderr("Unable to restore site application '$appName' for %s domain '$domainName' because accordable package will not be installed.\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
			&setRestoreFlags($baseNode, undef, $domainNode, {'restore' => 'false'});
			return undef;
		}

		# check SiteApp installation:
		# SiteAppPackages.(name, version, release) and SiteApp.(install_prefix)
		my $qstr = sprintf('SELECT sa.id, sa.install_prefix FROM SiteApps sa, SiteAppPackages sap WHERE sa.app_id=sap.id AND sa.dom_type=%s AND sap.id=%s AND sa.dom_id=%s',
							dbQuote($baseNode->getAttribute('dom_type')),
							dbQuote($pkgNode->getAttribute('real_id')),
							$subDomainName ? dbQuote($subDomainId) : dbQuote($domainId)
						);
		my $query = dbDoQuery($qstr);
		unless ($query) {
			printf_stderr("Unable to check collisions for site application '$appName' on %s domain '$domainName'\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
			&setRestoreFlags($baseNode, undef, $domainNode, {'restore' => 'false'});
			return undef;
		}
		my ($id, $install_prefix) = $query->fetchrow_array();
		if ($id && ($install_prefix eq $dumped_install_prefix)) {
			&setRestoreFlags($baseNode, $parentNode, $domainNode, {'exists' => 'true', 'restore' => 'false'});
			printf_stderr("Site application '$appName' is already installed for %s domain '$domainName'\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
			return 1;
		} elsif ($dumped_install_prefix && (-d $install_dir)) {
			&setRestoreFlags($baseNode, $parentNode, $domainNode, {'collision' => 'true', 'restore' => 'false'});
			printf_stderr("Can't restore Site application '$appName' because install_prefix ($dumped_install_prefix) directory already exists in vhost %s '$domainName'\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
			return 1;
		}

		# check SiteApp related resources
		foreach my $resRefNode ($baseNode->xql('./resources/*')) {
			my $type = u2a($resRefNode->getAttribute('type'));
			if ($type eq 'database') {
				foreach my $dbNode ($domainNode->xql('./data_bases[@id="'.$resRefNode->getAttribute('res_id').'"]')) {
					if ($dbNode->getAttribute('restore') eq 'false') {
						&setRestoreFlags($baseNode, undef, $domainNode, {'restore' => 'false'});
						printf_stderr("Can't restore Site application '$appName' for %s domain '$domainName' because related database '" . $dbNode->getAttribute('name') . "' will not be restored\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
						return 1;
					}
					if (($dbNode->getAttribute('exists') eq 'true') || ($dbNode->getAttribute('collision') eq 'true')) {
						&setRestoreFlags($baseNode, undef, $domainNode, {'collision' => 'true', 'restore' => 'false'});
						printf_stderr("Can't restore Site application '$appName' for %s domain '$domainName' because related database '" . $dbNode->getAttribute('name') . "' conflicts with some existing database\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
						return 1;
					}
				}
			} elsif ($type eq 'dbuser') {
				foreach my $dbUsrNode ($domainNode->xql('./data_bases/user[@type="db_users"]/login[@id="'.$resRefNode->getAttribute('res_id').'"]')) {
					if ($dbUsrNode->getAttribute('restore') eq 'false') {
						&setRestoreFlags($baseNode, undef, $domainNode, {'restore' => 'false'});
						printf_stderr("Can't restore Site application '$appName' for %s domain '$domainName' because user '" . $dbUsrNode->getAttribute('name') . "' of related database will not be restored\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
						return 1;
					}
					if (($dbUsrNode->getAttribute('exists') eq 'true') || ($dbUsrNode->getAttribute('collision') eq 'true')) {
						&setRestoreFlags($baseNode, undef, $domainNode, {'collision' => 'true', 'restore' => 'false'});
						printf_stderr("Can't restore Site application '$appName' for %s domain '$domainName' because user '" . $dbUsrNode->getAttribute('name') . "' of related database conflicts with some existing dtabase user\n", ($subDomainName ? "subdomain '$subDomainName' of " : ''));
						return 1;
					}
				}
			}
		}
	} elsif ($flags{RESTORE_MODE} eq 'RESTORE') {
		if (($baseNode->getAttribute('collision') eq 'true') || ($baseNode->getAttribute('restore') eq 'false')) {
			print_stderr("Site application restoration is skipped\n");
			return 1;
		}

		# check package existing in db
		unless ($pkgNode->getAttribute('real_id')) {
			print_stderr("Warning: site application is restored as hosting content, but will not be registered in control panel.\n");
			printf_stderr("Site application is not restored: info of accordable site application package does not exist in database\n");
			return undef;
		}

		# check content restoration
		unless ( -d $install_dir) {
			print_stderr("Unable to restore '$appName' because directory '$install_dir' does not exists\n");
			return undef;
		}

		# restore site application registration info
		dbDoQuery(sprintf('INSERT INTO SiteApps(dom_id, dom_type, install_prefix, htdocs_directory, app_id) values(%s,%s,%s,%s,%s)',
					$subDomainId ?  dbQuote($subDomainId) : dbQuote($domainId),
					dbQuote($baseNode->getAttribute('dom_type')),
					dbQuote($baseNode->getAttribute('install_prefix')),
					dbQuote($baseNode->getAttribute('htdocs_directory')),
					dbQuote($pkgNode->getAttribute('real_id'))
				));

		my $app_id = dbGetInsertId();

		# restore SiteApp related resources
		foreach my $resRefNode ($baseNode->xql('./resources/*')) {
			my $type = u2a($resRefNode->getAttribute('type'));
			if ($type eq 'database') {
				foreach my $dbNode ($domainNode->xql('./data_bases[@id="'.u2a($resRefNode->getAttribute('res_id')).'"]')) {
					my $db_real_id = $dbNode->getAttribute('real_id') + 0;
					if ($db_real_id <= 0) {
						printf_stderr("Error during site application restoration: related database '%s' does not exist\n", $dbNode->getAttribute('name'));
						next; #return undef;
					}
					dbDoQuery(sprintf('INSERT INTO SiteAppResources(app_id, type, res_id) values(%s,%s,%s)',
								dbQuote($app_id),
								dbQuote($resRefNode->getAttribute('type')),
								dbQuote($db_real_id)
							));
				}
			} elsif ($type eq 'dbuser') {
				foreach my $dbUsrNode ($domainNode->xql('./data_bases/user[@type="db_users"]/login[@id="'.u2a($resRefNode->getAttribute('res_id')).'"]')) {
					my $db_usr_real_id = $dbUsrNode->getAttribute('real_id') + 0;
					if ($db_usr_real_id <= 0) {
						printf_stderr("Error during site application restoration: related database user '%s' does not exist\n", $dbUsrNode->getAttribute('name'));
						next; #return undef;
					}
					dbDoQuery(sprintf('INSERT INTO SiteAppResources(app_id, type, res_id) values(%s,%s,%s)',
								dbQuote($app_id),
								dbQuote($resRefNode->getAttribute('type')),
								dbQuote($db_usr_real_id)
							));
				}
			}
		}

		# restore SiteApp parameters
		my $params_id = undef;
		foreach my $node ($baseNode->xql('./params/*')) {
			if ($params_id) {
				dbDoQuery(sprintf('INSERT INTO Parameters(id, parameter, value) values(%s,%s,%s)',
							dbQuote($params_id),
							dbQuote(u2a($node->getAttribute('name'))),
							dbQuote(u2a($node->getAttribute('value')))
						));
			} else {
				dbDoQuery(sprintf('INSERT INTO Parameters(parameter, value) values(%s,%s)',
							dbQuote(u2a($node->getAttribute('name'))),
							dbQuote(u2a($node->getAttribute('value')))
						));
				$params_id = dbGetInsertId();
				dbDoQuery("UPDATE SiteApps SET params_id='$params_id' WHERE id='$app_id'");
			}
		}
	}

	return 1;
}


sub setRestoreFlags
{
	my ($node, $parent_node, $domainNode, $flags) = @_;


	foreach my $i (keys %$flags) {
		$node->setAttribute($i, $$flags{$i});
	}

	if ($parent_node) {
		$parent_node->setAttribute('content_restore', 'false');
	} else {
		print_stderr("Warning: site application will be restored as hosting content, but will not be registered in control panel.\n");
	}
#	if ($parent_node && ($$flags{'collision'} eq 'true')) {
#		&markCollision($parent_node);
#		&markCollision($parent_node);
#	}

	# mark resources
	foreach my $resRefNode ($node->xql('./resources/*')) {
		my $type = u2a($resRefNode->getAttribute('type'));
		if ($type eq 'database') {
			foreach my $dbNode ($domainNode->xql('./data_bases[@id="'.$resRefNode->getAttribute('res_id').'"]')) {
				$dbNode->setAttribute('restore', 'false');
			}
		} elsif ($type eq 'dbuser') {
			foreach my $dbUsrNode ($domainNode->xql('./data_bases/user[@type="db_users"]/login[@id="'.$resRefNode->getAttribute('res_id').'"]')) {
				$dbUsrNode->setAttribute('restore', 'false');
				$dbUsrNode->getParentNode()->setAttribute('restore', 'false');
			}
		}
	}
}
